==> unity-chromium-extension_1.9.2.orig.tar.gz: unity_webapps_chromium-1.9.2/chromium-extension/background-page.js <==
/* Chromium Unity integration extension
 * 
 *   Copyright 2012 Canonical Ltd.
 *
 *   This program is free software: you can redistribute it and/or modify it 
 *   under the terms of the GNU General Public License version 3, as published 
 *   by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful, but 
 *   WITHOUT ANY WARRANTY; without even the implied warranties of 
 *   MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
 *   PURPOSE.  See the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License along 
 *   with this program.  If not, see <http://www.gnu.org/licenses/>.
 **/

(function () {
   
   // TODO extract based on logging settings
   var log = {
     info: function (msg) {
       if (msg) {
	 console.log ('Info: ' + msg);
       }
     }
     ,
     error: function (msg) {
       if (msg) {
	 console.log ('Error: ' + msg);
       }
     }
   };
   
   var plugin = document.getElementById ('unityChromiumExtensionId');
   if ( ! plugin) {
     log.error ("Unable to retrieve the Unity Chromium plugin");
     return;
   }
   var service = plugin.service_new ();
   if (! service) {
     log.error ("Unable to retrieve a connection to Unity Webapps service");
     return;
   }

   var loginListeners = [];

   var onLoginListenerRequest = function (request, sender, sendResponse) {
     if (request.action == "registerLoginListener") {
       log.info ("Login listener request from " + sender.id);
       loginListeners.push (sender.id);
     }
   };
   chrome.extension.onRequestExternal.addListener (onLoginListenerRequest);

   var onLoginEvent = function (request, sender, sendResponse) {
     if (request.action == "loginEvent") {
       log.info ("Login on " + request.domain + ": " + request.login);
       var len = loginListeners.length;
       for (var i = 0; i < len; i++) {
         var extensionId = loginListeners[i];
         log.info("Notifying login listener: " + extensionId);
         chrome.extension.sendRequest (extensionId, {
                                         type: "login",
                                         login: request.login,
                                         domain: request.domain
                                       });
       }
     }
   };
   chrome.extension.onRequest.addListener (onLoginEvent);

   var gatherUserScriptsDescriptions = function (plugin) {
     
     var repo = plugin.get_integration_scripts_repo ();
     return repo.getIntegrationScripts ();
   };

   var buildIntegrationScriptDB = function (plugin) {
     var db = [];
     
     var integrationScripts = null;
     try {
       integrationScripts = gatherUserScriptsDescriptions (plugin);
     }
     catch (e) {
       log.error ("Error while trying to gather user script descriptions: " + String(e));
       return db;
     }
     
     if ( ! integrationScripts) {
       return db;
     }
     
     log.info ("Found " + integrationScripts.length + " integration scripts");
     
     for (var i = 0; i < integrationScripts.length; ++i) {
       
       if ( ! integrationScripts[i]) {
	 log.error ('Error while going through the integration scripts: null for index ' + i);
	 continue;
       }
       
       var script = {};
       
       script.includes = integrationScripts[i].includes.split (';');
       script.includes = script.includes.map (
	 function (incl) {
	   var p = null;
	   if (incl && incl.length !== 0) {
	     try { p = new MatchPattern (incl); } catch (e) {};
	   }
	   return p;
	 }
       );
       script.includes = script.includes.filter (function (elt) { return null !== elt; });
       
       script.name = integrationScripts[i].name + "";
       script.content = integrationScripts[i].content + "";
       script.requires = integrationScripts[i].requires + "";
       
       // TODO: mmmh keep a ref?
       script.ref = integrationScripts[i];
       
       db.push (script);
     }
     
     return db;
   };
   var db = buildIntegrationScriptDB (plugin);
   
   
   // performs a match on the list of loaded integration scripts
   // given a url.
   var matchesIntegrationScripts = function (url, scripts) {
     function testMatch(matchPattern) {
       return matchPattern.doMatch(url);
     };
     
     var matches = [];
     for (var i = 0; i < scripts.length; ++i) {
       var script = scripts[i];
       try {
	 if (script.includes && script.includes.some (testMatch)) {
	   matches.push(script);
	 }
       }
       catch (err)
       {}
     }
     return matches;
   };

   // handles tab change & activation code
   chrome.tabs.onActivated.addListener (
     function (info) {
       // TODO: pretty bad ... assumed that all tabs are interested
       chrome.windows.getAll ({populate: true}
			      , function (windows) {
				for (var winIdx in windows) {
				  var tabs = windows[winIdx].tabs;
				  for (var tabIdx in tabs) {
				    var tabId = tabs[tabIdx].id;
				    chrome.tabs.sendRequest (tabId
							     , {method: "on_tab_active_changed"
								, tabId: info.tabId
							       }
							     , function (response) {});
				  } // for (var tabIdx
				} // for (var winIdx
			      });
     }
   );

   // TODO move elsewhere
   // TODO check that's it's fine (it should), since contrary to
   //      FF this is being called on the same environment every time (background page)
   function toDataURL (uri, callback, sx, sy, sw, sh) {
     if (uri.match(/^\s{0,}file:/i))
       return;
     
     if (uri.match(/^\s{0,}https:/i))
       return;
     
     var self = this;
     var canvas = document.createElementNS("http://www.w3.org/1999/xhtml", "canvas");
     var img = new window.Image();
     
     img.onload = function() {
       var width = img.width, height = img.height;

       var hasExtraArgs = sx || sx === 0;
       if (hasExtraArgs) {
	 width = sw;
	 height = sh;
       }

       var dx  = 0, dy = 0;

       if (width > height)
	 dy = width - height;
       else
	 dx = height - width;

       canvas.width = width + dx;
       canvas.height = height + dy;
       
       var ctx = canvas.getContext('2d');
       if (hasExtraArgs)
	 ctx.drawImage(img, sx, sy, sw, sh, dx / 2, dy / 2, width, height);
       else
	 ctx.drawImage(img, dx / 2, dy / 2);
       
       callback (true, canvas.toDataURL());
     };
     img.src = uri;
   };
   
   
   ////////////////////////////////////////
   // 
   // main request handler
   // 
   ///////////////////////////////////////
   
   // list of callback that are to be called asynchronously
   //  per tab. Used in the user integration resquests contex.
   var user_integration_callbacks = {};
   
   var contentScriptsRequestHandler = function (request, sender, callback) {
     
     var addIntegrationCallbackFor = function (integrationId, callback, message, details) {
       
       user_integration_callbacks[integrationId] = {callback: callback
						    , message: message
                                                    , details: details
						   };
     };
     var getDataIfAnyFor = function (integrationId) {
       
       if (user_integration_callbacks[integrationId] === undefined) {
	 return "";
       }
       return { message: user_integration_callbacks[integrationId].message,
                details: user_integration_callbacks[integrationId].details };
     };
     var invokeAndRemoveCallbackIfAnyFor = function (integrationId, arguments) {
       
       if (user_integration_callbacks[integrationId] === undefined) {
	 return;
       }
       var callback = user_integration_callbacks[integrationId].callback;
       user_integration_callbacks[integrationId] = undefined;
       if (typeof(callback) === 'function') {
	 callback(arguments);
       }
     };

     var handlers = {
       get_unity_script: function (request, sender, callback) {
	 
	 var results = [];
	 var scripts = matchesIntegrationScripts (request.url, db);
	 if (scripts) {
           for (var i = 0; i < scripts.length; ++i) {
             var result = {
	       userscript: null
	       , requires: null
	     };
             result.userscript = scripts[i].content;
	     if (scripts[i].requires && scripts[i].requires.length !== 0) {
	       result.requires = scripts[i].requires;
	     }
	     results.push(result);
           }
	 }
	 callback (results);
       }
       ,
       get_base_domain: function (request, sender, callback) {
	 
	 chrome.tld.getBaseDomain (request.url, callback);
       }
       , 
       get_message_for_user_integration: function (request, sender, callback) {

	 callback (getDataIfAnyFor (request.tabId));
       }
       ,
       add_integrationscript_for_tabid: function (request, sender, callback) {
         
         addIntegrationScriptForTab(sender.tab.id, request.name, request.domain, request.interest_id);
       }
       ,
       on_user_integration_result: function (request, sender, callback) {
	 
	 invokeAndRemoveCallbackIfAnyFor (request.tabId, request);
       }
       ,
       request_user_integration: function (request, sender, callback) {
	 
	 addIntegrationCallbackFor (sender.tab.id, callback, request.message, request.details);
	 
	 chrome.infobars.show ({tabId: sender.tab.id, path: "infobar.html"});
       }
       , 
       take_snapshot: function (request, sender, callback) {
	 
	 chrome.tabs.captureVisibleTab (sender.tab.windowId
					, null
					, function (dataURL) {
					  callback (dataURL);
					});
       }
       , 
       get_data_url_from: function (request, sender, callback) {
	 
	 toDataURL (request.url
		    , function (flag, dataUrl) {
		      
		      callback ({flag: flag, dataUrl: dataUrl});
		    });
       }
       , 
       get_extension_settings: function (request, sender, callback) {
	 
	 var settings = {
	   logging: false,
           incognito: sender.tab.incognito
	 };
	 
	 try {
	   if (window.localStorage) {
	     settings.logging = localStorage['logging'];
	   }
	 }
	 catch (e) {
	   log.error ("Error while trying to retrieve logging information: " + str(e));
	 }
	 
	 callback (settings);
       }
       , 
       get_tab_info: function (request, sender, callback) {
	 
	 callback ({tabId: sender.tab.id, windowId: sender.tab.windowId});
       }
       , 
       kill_this_tab: function (request, sender, callback) {
	 
	 chrome.tabs.remove ([request.tabId], function () {});
       }
       , 
       raise_this_tab: function (request, sender, callback) {
	 
	 // bring the window & the tab to the front
	 chrome.windows.update (request.windowId
				, {focused: true}
				, function (window) {
				  chrome.tabs.update (request.tabId
						      , {active: true, highlighted: true});
				});
       }
       , 
       is_selected: function (request, sender, callback) {
	 
	 chrome.tabs.getCurrent (
	   function (tab) {
	     if (tab) {
	       callback ({isSelected: tab.id === request.tabId});
	     }
	   }
	 );
       }
       
     }; // handlers
     
     
     // validate request
     if ( ! request  || ! request.method) {
       callback ({error: "Invalid request structure"});
       return;
     }
     if ( ! sender) {
       callback ({error: "Invalid sender"});
       return;
     }
     if ( typeof (request.method) != 'string' || request.method.length == 0) {
       callback ({error: "Invalid request method"});
       return;
     }
     
     log.info ('Got request: ' + request.method + ', ' + sender.tab.id);
     
     var handler = handlers [request.method];
     
     if (handler !== undefined && typeof(handler) == 'function') {
       
       log.info ('Calling handler for: ' + request.method);
       
       handler (request, sender, callback);
     }
   };
   
   // main event handler
   chrome.extension.onRequest.addListener (contentScriptsRequestHandler);
   
   // tab management handling
   var integration_script_tabs = {};
   var addIntegrationScriptForTab = function (tabId, name, domain, interest_id) {
     if (integration_script_tabs[tabId] !== undefined) {
       log.info('Overriding tab information for existing tab: ' + tabId);
     }
     log.info ('Integrating tab with name: ' + name);
     integration_script_tabs[tabId] = {name: name, domain: domain, interest_id: interest_id};
   };
   var removeIntegrationScriptForTab = function (tabId) {
     if (integration_script_tabs[tabId] !== undefined) {
       integration_script_tabs[tabId] = undefined;
     }
   };
   chrome.tabs.onRemoved.addListener (
     function (tabId, removeInfo) {
       var info = integration_script_tabs[tabId];
       removeIntegrationScriptForTab (tabId);
       if (info !== undefined) {
         log.info('Destroying interest: ' + info.name + ', ' + info.domain + ', ' + info.interest_id);
         try {
           plugin.service_destroy_interest_for_context (service
                                                        , info.name
                                                        , info.domain
                                                        , info.interest_id
                                                        , 1);
         }
         catch (e) {
           log.error('Error while trying to destroy interest for tab ' + tabId + ', ' + e);
         }
       }
     }
   );

   // misc (to remove)
   chrome.tabs.onCreated.addListener (
     function (tab) {
       var script = matchesIntegrationScripts (tab.url, db);
       if (script.length !== 0) {
	 log.info ("New tab created for URL: " + tab.url + ", matches an integration script");
       }
     }
   );

 }) ();

==> unity-chromium-extension_1.9.3.orig.tar.gz: unity_webapps_chromium-1.9.3/chromium-extension/background-page.js <==
/* Chromium Unity integration extension
 * 
 *   Copyright 2012 Canonical Ltd.
 *
 *   This program is free software: you can redistribute it and/or modify it 
 *   under the terms of the GNU General Public License version 3, as published 
 *   by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful, but 
 *   WITHOUT ANY WARRANTY; without even the implied warranties of 
 *   MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
 *   PURPOSE.  See the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License along 
 *   with this program.  If not, see <http://www.gnu.org/licenses/>.
 **/

(function () {
   
   // TODO extract based on logging settings
   var log = {
     info: function (msg) {
       if (msg) {
	 console.log ('Info: ' + msg);
       }
     }
     ,
     error: function (msg) {
       if (msg) {
	 console.log ('Error: ' + msg);
       }
     }
   };


   // connection & init w/ plugin NPAPI layer
   var plugin = document.getElementById ('unityChromiumExtensionId');
   if ( ! plugin) {
     log.error ("Unable to retrieve the Unity Chromium plugin");
     return;
   }
   var service = plugin.service_new ();
   if (! service) {
     log.error ("Unable to retrieve a connection to Unity Webapps service");
     return;
   }


   /////////////////////////////////////////////////////////
   // 
   // Handle login & connection to online-accounts project's
   // extension
   // 
   ////////////////////////////////////////////////////////
   var loginListeners = [];
   var onLoginListenerRequest = function (request, sender, sendResponse) {
     if (request.action == "registerLoginListener") {
       log.info ("Login listener request from " + sender.id);
       loginListeners.push (sender.id);
     }
   };
   chrome.extension.onRequestExternal.addListener (onLoginListenerRequest);

   var onLoginEvent = function (request, sender, sendResponse) {
     if (request.action == "loginEvent") {
       log.info ("Login on " + request.domain + ": " + request.login);
       var len = loginListeners.length;
       for (var i = 0; i < len; i++) {
         var extensionId = loginListeners[i];
         log.info("Notifying login listener: " + extensionId);
         chrome.extension.sendRequest (extensionId, {
                                         type: "login",
                                         login: request.login,
                                         domain: request.domain
                                       });
       }
     }
   };
   chrome.extension.onRequest.addListener (onLoginEvent);


   /////////////////////////////////////////////////////////
   // 
   // Extract installed userscripts & webapps info
   // 
   ////////////////////////////////////////////////////////
   var gatherUserScriptsDescriptions = function (plugin) {
     
     var repo = plugin.get_integration_scripts_repo ();
     return repo.getIntegrationScripts ();
   };

   var buildIntegrationScriptDB = function (plugin) {
     var db = [];
     
     var integrationScripts = null;
     try {
       integrationScripts = gatherUserScriptsDescriptions (plugin);
     }
     catch (e) {
       log.error ("Error while trying to gather user script descriptions: " + String(e));
       return db;
     }
     
     if ( ! integrationScripts) {
       return db;
     }
     
     log.info ("Found " + integrationScripts.length + " integration scripts");
     
     for (var i = 0; i < integrationScripts.length; ++i) {
       
       if ( ! integrationScripts[i]) {
	 log.error ('Error while going through the integration scripts: null for index ' + i);
	 continue;
       }
       
       var script = {};
       
       script.includes = integrationScripts[i].includes.split (';');
       script.includes = script.includes.map (
	 function (incl) {
	   var p = null;
	   if (incl && incl.length !== 0) {
	     try { p = new MatchPattern (incl); } catch (e) {};
	   }
	   return p;
	 }
       );
       script.includes = script.includes.filter (function (elt) { return null !== elt; });
       
       script.name = integrationScripts[i].name + "";
       script.content = integrationScripts[i].content + "";
       script.requires = integrationScripts[i].requires + "";
       
       // TODO: mmmh keep a ref?
       script.ref = integrationScripts[i];
       
       db.push (script);
     }
     
     return db;
   };
   var db = buildIntegrationScriptDB (plugin);
   
   
   /**
    * Performs a match on the list of loaded integration scripts
    * given a url.
    *
    */
   var matchesIntegrationScripts = function (url, scripts) {
     function testMatch(matchPattern) {
       return matchPattern.doMatch(url);
     };
     
     var matches = [];
     for (var i = 0; i < scripts.length; ++i) {
       var script = scripts[i];
       try {
	 if (script.includes && script.includes.some (testMatch)) {
	   matches.push(script);
	 }
       }
       catch (err)
       {}
     }
     return matches;
   };


   /**
    * Handles tab change & activation code.
    * TODO(FIXME): better window tracking for active windows & tabs
    *   -> there are some glitches when a given webapp tab is moved around:
    *      1. open chromeless webapp w/ webapp (A)
    *      2. open normal window webapp (B) & unrelated other tab in same window
    *      3. undock (B) and dock it in (A): issues w/ activation (in laucnher)
    *      4. close the "first webapp" (A) -> actiating the webapp icon now
    *         refers back to first window, might be bc of BAMF window tracking
    *
    */
   chrome.tabs.onActivated.addListener (
     function (info) {
       // TODO(FIXME): pretty bad ... assumed that all tabs are interested
       chrome.windows.getAll (
         {populate: true}
	 , function (windows) {
	   for (var winIdx in windows) {
	     var tabs = windows[winIdx].tabs;
	     for (var tabIdx in tabs) {
	       var tabId = tabs[tabIdx].id;
	       chrome.tabs.sendRequest (tabId
					, {method: "on_tab_active_changed"
					   , tabId: info.tabId
					  }
					, function (response) {});
	     } // for (var tabIdx
	   } // for (var winIdx
	 });
     }
   );

   // TODO move elsewhere
   // TODO check that's it's fine (it should), since contrary to
   //      FF this is being called on the same environment every time (background page)
   function toDataURL (uri, callback, sx, sy, sw, sh) {
     if (uri.match(/^\s{0,}file:/i))
       return;
     
     if (uri.match(/^\s{0,}https:/i))
       return;
     
     var self = this;
     var canvas = document.createElementNS("http://www.w3.org/1999/xhtml", "canvas");
     var img = new window.Image();
     
     img.onload = function() {
       var width = img.width, height = img.height;

       var hasExtraArgs = sx || sx === 0;
       if (hasExtraArgs) {
	 width = sw;
	 height = sh;
       }

       var dx  = 0, dy = 0;

       if (width > height)
	 dy = width - height;
       else
	 dx = height - width;

       canvas.width = width + dx;
       canvas.height = height + dy;
       
       var ctx = canvas.getContext('2d');
       if (hasExtraArgs)
	 ctx.drawImage(img, sx, sy, sw, sh, dx / 2, dy / 2, width, height);
       else
	 ctx.drawImage(img, dx / 2, dy / 2);
       
       callback (true, canvas.toDataURL());
     };
     img.src = uri;
   };
   
   
   ////////////////////////////////////////
   // 
   // main request handler
   // 
   ///////////////////////////////////////
   
   // list of callback that are to be called asynchronously
   //  per tab. Used in the user integration resquests contex.
   var user_integration_callbacks = {};
   
   /**
    * Handles & responds to content script requests.
    * 
    */
   var contentScriptsRequestHandler = function (request, sender, callback) {
     
     var addIntegrationCallbackFor = function (integrationId, callback, message, details) {
       
       user_integration_callbacks[integrationId] = {callback: callback
						    , message: message
                                                    , details: details
						   };
     };
     var getDataIfAnyFor = function (integrationId) {
       
       if (user_integration_callbacks[integrationId] === undefined) {
	 return "";
       }
       return { message: user_integration_callbacks[integrationId].message,
                details: user_integration_callbacks[integrationId].details };
     };
     var invokeAndRemoveCallbackIfAnyFor = function (integrationId, arguments) {
       
       if (user_integration_callbacks[integrationId] === undefined) {
	 return;
       }
       var callback = user_integration_callbacks[integrationId].callback;
       user_integration_callbacks[integrationId] = undefined;
       if (typeof(callback) === 'function') {
	 callback(arguments);
       }
     };

     var handlers = {
       get_unity_script: function (request, sender, callback) {
	 
	 var results = [];
	 var scripts = matchesIntegrationScripts (request.url, db);
	 if (scripts) {
           for (var i = 0; i < scripts.length; ++i) {
             var result = {
	       userscript: null
	       , requires: null
	     };
             result.userscript = scripts[i].content;
	     if (scripts[i].requires && scripts[i].requires.length !== 0) {
	       result.requires = scripts[i].requires;
	     }
	     results.push(result);
           }
	 }
	 callback (results);
       }
       ,
       get_base_domain: function (request, sender, callback) {
	 
	 chrome.tld.getBaseDomain (request.url, callback);
       }
       , 
       get_message_for_user_integration: function (request, sender, callback) {

	 callback (getDataIfAnyFor (request.tabId));
       }
       ,
       add_integrationscript_for_tabid: function (request, sender, callback) {
         
         addIntegrationScriptForTab(sender.tab.id
                                    , request.name
                                    , request.domain
                                    , request.interest_id);
       }
       ,
       on_user_integration_result: function (request, sender, callback) {
	 
	 invokeAndRemoveCallbackIfAnyFor (request.tabId, request);
       }
       ,
       request_user_integration: function (request, sender, callback) {
	 
	 addIntegrationCallbackFor (sender.tab.id, callback, request.message, request.details);
	 
	 chrome.infobars.show ({tabId: sender.tab.id, path: "infobar.html"});
       }
       , 
       take_snapshot: function (request, sender, callback) {
	 
	 chrome.tabs.captureVisibleTab (sender.tab.windowId
					, null
					, function (dataURL) {
					  callback (dataURL);
					});
       }
       , 
       get_data_url_from: function (request, sender, callback) {
	 
	 toDataURL (request.url
		    , function (flag, dataUrl) {
		      
		      callback ({flag: flag, dataUrl: dataUrl});
		    });
       }
       , 
       get_extension_settings: function (request, sender, callback) {
	 
	 var settings = {
	   logging: false,
           incognito: sender.tab.incognito
	 };
	 
	 try {
	   if (window.localStorage) {
	     settings.logging = localStorage['logging'];
	   }
	 }
	 catch (e) {
	   log.error ("Error while trying to retrieve logging information: " + str(e));
	 }
	 
	 callback (settings);
       }
       , 
       get_tab_info: function (request, sender, callback) {
	 
	 callback ({tabId: sender.tab.id, windowId: sender.tab.windowId});
       }
       , 
       kill_this_tab: function (request, sender, callback) {
	 
	 chrome.tabs.remove ([request.tabId], function () {});
       }
       , 
       raise_this_tab: function (request, sender, callback) {
	 
	 // bring the window & the tab to the front
	 chrome.windows.update (request.windowId
				, {focused: true}
				, function (window) {
				  chrome.tabs.update (request.tabId
						      , {active: true, highlighted: true});
				});
       }
       , 
       is_selected: function (request, sender, callback) {
	 
	 chrome.tabs.getCurrent (
	   function (tab) {
	     if (tab) {
	       callback ({isSelected: tab.id === request.tabId});
	     }
	   }
	 );
       }
     }; // handlers
     
     
     // validate request
     if ( ! request  || ! request.method) {
       callback ({error: "Invalid request structure"});
       return;
     }
     if ( ! sender) {
       callback ({error: "Invalid sender"});
       return;
     }
     if ( typeof (request.method) != 'string' || request.method.length == 0) {
       callback ({error: "Invalid request method"});
       return;
     }
     
     log.info ('Got request: ' + request.method + ', ' + sender.tab.id);
     
     var handler = handlers [request.method];
     
     if (handler !== undefined && typeof(handler) == 'function') {
       
       log.info ('Calling handler for: ' + request.method);
       
       handler (request, sender, callback);
     }
   }; // var contentScriptsRequestHandler =

   // Main event handler and communication link
   //   w/ content scripts
   chrome.extension.onRequest.addListener (contentScriptsRequestHandler);


   ///////////////////////////////////////////////
   // 
   // window/tab management handling
   //
   ///////////////////////////////////////////////

   // Every tab that's being integrated as a webapp (chromeless or not)
   //  actually registers here. This allows tabs life cycle to be managed
   //  a bit when removed, or in chromeless mode when moved around.
   var integration_script_tabs = {};
   var addIntegrationScriptForTab = function (tabId
                                              , name
                                              , domain
                                              , interest_id) {
     if (integration_script_tabs[tabId] !== undefined) {
       log.info('Overriding tab information for existing tab: ' + tabId);
     }
     log.info ('Integrating tab with name: ' + name + ', for domain ' + domain);
     integration_script_tabs[tabId] = {name: name
                                       , domain: domain
                                       , interest_id: interest_id};
   };
   var removeIntegrationScriptForTab = function (tabId) {
     if (integration_script_tabs[tabId] !== undefined) {
       integration_script_tabs[tabId] = undefined;
     }
   };
   chrome.tabs.onRemoved.addListener (
     function (tabId, removeInfo) {
       var info = integration_script_tabs[tabId];
       removeIntegrationScriptForTab (tabId);
       if (info !== undefined) {
         log.info('Destroying interest: ' + info.name + ', ' + info.domain + ', ' + info.interest_id);
         try {
           plugin.service_destroy_interest_for_context (service
                                                        , info.name
                                                        , info.domain
                                                        , info.interest_id
                                                        , 1);
         }
         catch (e) {
           log.error('Error while trying to destroy interest for tab ' + tabId + ', ' + e);
         }
       }
     }
   );


   ///////////////////////////////////////////////////////////////////////
   // 
   // Window management related functions. In chromeless mode, we have specific
   //  rules for tab management to make webapps feel more "native" than plain
   //  web applications.
   // 
   ///////////////////////////////////////////////////////////////////////

   // BIG TODO: due to async specific of chrome api, all this is sensitive to timing
   //  issues / race conditions, so besides adding support for this directly in
   //  Chromium, one way to mitigate this would be to add support for the 'ischromeless'
   //  field in the tabs info structure retrieved from chrome API.
   // 

   /**
    * Validates the presence of a given tab in a chromeless window. The validation
    *  occurs based on specific rules:
    *    - for a given chromeless window, only tabs related to a gievn webapp should
    *       be allowed,
    *    - tabs can be 'undocked' but not 'external' tab can be docked in,
    * 
    * TODO: right now the scheme is pretty naive and brittle, fix that
    */
   var validateChromelessTabForWindowId =
     function (windowId, tabId, url, onInvalidTab) {
       // TODO bail out right away?
       var continuation =
         (onInvalidTab && typeof(onInvalidTab) == "function")
         ? onInvalidTab
         : function (tabId) { log.error ("onInvalidTab callback is invalid"); };

       // TODO use the new chrome.extension.getViews ({type: "tab"}) extension?
       // TODO(bis): mmmh one level of indirection again?
       chrome.windows.get(
         windowId
         , {populate: true}
         , function (win) {
           if (win.tabs.length == 1) {
             return;
           }
           var diff = win.tabs.some (
             function (curtab) {
               if (curtab.id == tabId) {
                 return false;
               }
               // TODO: usage of integration_script_tabs structure is bad
               var infos = integration_script_tabs[curtab.id];
               return infos
                 && infos.domain
                 && url.indexOf(infos.domain) == -1;
             }
           );
           if (diff) {
             // the tab corresponding to tabId
             // shoudldn't be there
             continuation(tabId);
           }
         }
       );
     }; // ) {

   chrome.tabs.onCreated.addListener(
     function(tab) {
       if (tab && tab.url) {
         chrome.extension.isChromelessWindow(
           tab.id
           , function (isChromeless) {
             if (!isChromeless) {
               return;
             }
             validateChromelessTabForWindowId(
               tab.windowId
               , tab.id
               , tab.url
               , function (tabId) {
                   chrome.windows.create({tabId: tab.id});
               }
             );
           } // function (isChromeless) {
         ); // chrome.extension.isChromelessWindow
       } // if (tab && tab.url) {
     }
   );
   chrome.tabs.onUpdated.addListener (
     function(tabId, changeInfo, tab) {
       chrome.extension.isChromelessWindow(
           tabId
         , function (isChromeless) {
             if (!isChromeless) {
               return;
             }
             if (changeInfo && changeInfo.url) {
               var infos = integration_script_tabs[tabId];
               if (infos &&
                   infos.domain &&
                   changeInfo.url.indexOf(infos.domain) == -1) {
                 chrome.windows.create({tabId: tabId});
               }
             }
           } // function (isChromeless)
       ); // chrome.extension.isChromelessWindow
     }
   );
   chrome.tabs.onAttached.addListener(
     function(tabId, attachInfo) {
       chrome.extension.isChromelessWindow(
           tabId
         , function (isChromeless) {
             if (!isChromeless) {
               return;
             }
             // retrieve info about the given tab
             // TODO: this get tiring bc of the sensitivity to
             //  race conditions (or equivalent) created by request lags
             chrome.tabs.get (
               tabId
               , function (tab) {
                   // Finally perform the actual validation or the
                   // new tab that got docked in
                   validateChromelessTabForWindowId(
                     attachInfo.newWindowId
                     , tabId
                     , tab.url
                     , function (tabId) {
                       chrome.windows.create({tabId: tab.id});
                     }
                   );
                 } // function (tab) {
             ); // chrome.tabs.get
         } // function (isChromeless) {
       ); // chrome.extension.isChromelessWindow
     }
   ); // tabs.onAttached

 }) ();
